# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import PGUtils
from Game import GameState
import random, itertools

# in grid, x+MAX_PIECES means the pair has been found, x+2*MAX_PIECES is the uncovered potential match
MAX_PIECES = 1000

class ConcentrationGameState(GameState):
    def __init__(self, dimensions=None, params=None, players=2, reset=True):
        GameState.__init__(self, dimensions, params, 1, reset)
        
    def reset(self):
        GameState.reset(self)
        # assign random pairs
        values = range(self.grid.size//2) * 2
        random.shuffle(values)
        for pos,val in itertools.izip(self.itercoords(), values):
            self.grid[pos] = val
            
    def player_score(self, pnum):
        if pnum!=1: return 0
        return sum(1 for val in self.itervalues() if MAX_PIECES<=val<2*MAX_PIECES)
                
    def uncovered_position(self):
        # find the (presumably one or zero) position with value >2*MAX_PIECES
        return PGUtils.first_item(pos for pos,val in self.enumerate_all() if val>=2*MAX_PIECES)
        
    def positions_match(self, pos1, pos2):
        return pos1 and pos2 and self.grid[pos1]%MAX_PIECES==self.grid[pos2]%MAX_PIECES
                
    def successors(self):
        ucpos = self.uncovered_position()
        
        for pos in self.itercoords():
            if self.grid[pos]<1000:
                nextgrid = self.grid.copy()
                if ucpos:
                    if self.positions_match(pos, ucpos):
                        # match, uncover both
                        nextgrid[pos] = nextgrid[ucpos] = nextgrid[pos] + MAX_PIECES
                    else:
                        # no match, cover ucpos
                        nextgrid[ucpos] = nextgrid[ucpos] - 2*MAX_PIECES
                else:
                    # pos becomes the potential match for the next move
                    nextgrid[pos] = nextgrid[pos] + 2*MAX_PIECES
                yield ( (pos,), self.next_state_with_grid(nextgrid) )
                
    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        ucpos = self.uncovered_position()
        # fixme: uncover_anim produces errors if duration=0
        uncover_anim = PGPieceAnimation.remove_animation(move[0], duration=0.001)
        reveal_anim = PGPieceAnimation.new_piece_animation(move[0], self.grid[move[0]]+MAX_PIECES, duration=0)
        animations = [uncover_anim, reveal_anim]
        if ucpos and not self.positions_match(ucpos, move[0]):
            # failed match, keep them around for a second with a dummy animation
            animations.append(PGPieceAnimation.move_animation(ucpos, ucpos, duration=1.0))
        return PGPieceAnimation.AnimationGroup(animations)
            
                
